@@ -4,7 +4,6 @@ from biopsykit.protocols.base import BaseProtocol
 from biopsykit.protocols.car import CAR
 from biopsykit.protocols.cft import CFT
 from biopsykit.protocols.mist import MIST
-from biopsykit.protocols.stroop import Stroop
 from biopsykit.protocols.tsst import TSST
 
-__all__ = ["BaseProtocol", "CFT", "CAR", "MIST", "TSST", "Stroop", "plotting"]
+__all__ = ["BaseProtocol", "CFT", "CAR", "MIST", "TSST", "plotting"]
