@@ -1,29 +1,24 @@
 """Module for processing ECG data."""
-from typing import Optional, Dict, Tuple, Union, Sequence, Callable, List
+from typing import Callable, Dict, List, Optional, Sequence, Tuple, Union
 
 import neurokit2 as nk
 import numpy as np
 import pandas as pd
-
 from scipy.stats import iqr
 from tqdm.auto import tqdm
 
 from biopsykit.signals._base import _BaseProcessor
+from biopsykit.utils.array_handling import find_extrema_in_radius, remove_outlier_and_interpolate, sanitize_input_1d
 from biopsykit.utils.datatype_helper import (
-    HeartRatePhaseDict,
     EcgRawDataFrame,
     EcgResultDataFrame,
+    HeartRatePhaseDict,
     RPeakDataFrame,
+    _EcgResultDataFrame,
+    _RPeakDataFrame,
     is_ecg_raw_dataframe,
     is_ecg_result_dataframe,
     is_r_peak_dataframe,
-    _EcgResultDataFrame,
-    _RPeakDataFrame,
-)
-from biopsykit.utils.array_handling import (
-    find_extrema_in_radius,
-    remove_outlier_and_interpolate,
-    sanitize_input_1d,
 )
 
 __all__ = ["EcgProcessor"]
@@ -466,15 +461,18 @@ class EcgProcessor(_BaseProcessor):
             List containing the outlier correction methods to be applied.
             Pass ``None`` to not apply any outlier correction, ``all`` to apply all available outlier correction
             methods. See :meth:`~biopsykit.signals.ecg.EcgProcessor.outlier_corrections` to get a list of possible
-            outlier correction methods. Default: ``all``
+            outlier correction methods.
+            Default: ``all``
         outlier_params: dict, optional
             Dict of parameters to be passed to the outlier correction methods or ``None``
             to use default parameters (see :meth:`~biopsykit.signals.ecg.EcgProcessor.outlier_params_default`
-            for more information). Default: ``None``
+            for more information).
+            Default: ``None``
         imputation_type: str, optional
             Method for outlier imputation: ``linear`` for linear interpolation between the RR intervals before and
             after R peak outlier, or ``moving_average`` for average value of the
-            10 preceding and 10 succeding RR intervals. Default: ``None`` (corresponds to ``moving_average``)
+            10 preceding and 10 succeding RR intervals.
+            Default: ``None`` (corresponds to ``moving_average``)
         sampling_rate : float, optional
             Sampling rate of recorded data in Hz. Not needed if ``ecg_processor`` is supplied as parameter.
             Default: 256
