@@ -4,7 +4,7 @@ For user facing type declarations, please see :py:func:`biopsykit.utils.datatype
 """
 
 from pathlib import Path
-from typing import Hashable, TypeVar, Union
+from typing import Hashable, Sequence, TypeVar, Union
 
 import numpy as np
 import pandas as pd
@@ -13,4 +13,5 @@ _Hashable = Union[Hashable, str]
 
 path_t = TypeVar("path_t", str, Path)  # pylint:disable=invalid-name
 arr_t = TypeVar("arr_t", pd.DataFrame, pd.Series, np.ndarray)  # pylint:disable=invalid-name
+str_t = TypeVar("str_t", str, Sequence[str])  # pylint:disable=invalid-name
 T = TypeVar("T")
